@@ -21,6 +21,8 @@
 #include <ogmrip-plugin.h>
 #include <ogmrip-fs.h>
 
+#include <ogmjob-queue.h>
+
 #include <glib/gi18n-lib.h>
 #include <glib/gstdio.h>
 
@@ -47,35 +49,42 @@ static GType ogmrip_oggz_get_type (void);
 static gint  ogmrip_oggz_run      (OGMJobSpawn *spawn);
 
 static gboolean have_oggz_merge = FALSE;
+static gboolean have_kate_enc   = FALSE;
 
 static void
-ogmrip_oggz_append_audio_file (OGMRipContainer *oggz, const gchar *filename, GPtrArray *argv)
-{
-  struct stat buf;
-
-  if (g_stat (filename, &buf) == 0 && buf.st_size > 0)
-    g_ptr_array_add (argv, g_strdup (filename));
-}
-
-static void
-ogmrip_oggz_foreach_audio (OGMRipContainer *oggz, 
+ogmrip_oggz_foreach_audio_stream (OGMRipContainer *oggz, 
     OGMRipCodec *codec, guint demuxer, gint language, GPtrArray *argv)
 {
-  const gchar *input;
+  gchar *input;
 
-  input = ogmrip_codec_get_output (codec);
-  ogmrip_oggz_append_audio_file (oggz, input, argv);
+  input = g_strdup (ogmrip_codec_get_output (codec));
+  g_ptr_array_add (argv, input);
 }
 
 static void
-ogmrip_oggz_foreach_audio_file (OGMRipContainer *oggz, OGMRipFile *file, GPtrArray *argv)
+ogmrip_oggz_foreach_subp_stream (OGMRipContainer *oggz,
+    OGMRipCodec *codec, guint demuxer, gint language, GPtrArray *argv)
+{
+  gchar *input;
+
+  input = g_strconcat (ogmrip_codec_get_output (codec), ".ogg", NULL);
+  g_ptr_array_add (argv, input);
+}
+
+static void
+ogmrip_oggz_foreach_file (OGMRipContainer *oggz, OGMRipFile *file, GPtrArray *argv)
 {
   gchar *filename;
 
   filename = ogmrip_file_get_filename (file);
-  if (filename && ogmrip_file_get_type (file) == OGMRIP_FILE_TYPE_AUDIO)
-    ogmrip_oggz_append_audio_file (oggz, filename, argv);
-  g_free (filename);
+  if (filename)
+  {
+    gint type = ogmrip_file_get_type (file);
+    if (type == OGMRIP_FILE_TYPE_AUDIO || (type == OGMRIP_FILE_TYPE_SUBP && have_kate_enc))
+      g_ptr_array_add (argv, filename);
+    else
+      g_free (filename);
+  }
 }
 
 static gchar **
@@ -100,9 +109,41 @@ ogmrip_oggz_command (OGMRipContainer *oggz)
   g_ptr_array_add (argv, g_strdup (filename));
 
   ogmrip_container_foreach_audio (oggz, 
-      (OGMRipContainerCodecFunc) ogmrip_oggz_foreach_audio, argv);
+      (OGMRipContainerCodecFunc) ogmrip_oggz_foreach_audio_stream, argv);
+
+  if (have_kate_enc)
+    ogmrip_container_foreach_subp (oggz,
+        (OGMRipContainerCodecFunc) ogmrip_oggz_foreach_subp_stream, argv);
+
   ogmrip_container_foreach_file (oggz, 
-      (OGMRipContainerFileFunc) ogmrip_oggz_foreach_audio_file, argv);
+      (OGMRipContainerFileFunc) ogmrip_oggz_foreach_file, argv);
+
+  g_ptr_array_add (argv, NULL);
+
+  return (gchar **) g_ptr_array_free (argv, FALSE);
+}
+
+static gchar **
+ogmrip_kate_command (const gchar *input)
+{
+  GPtrArray *argv;
+  gchar *output;
+
+  argv = g_ptr_array_new ();
+  g_ptr_array_add (argv, g_strdup ("kateenc"));
+  g_ptr_array_add (argv, g_strdup ("-t"));
+  g_ptr_array_add (argv, g_strdup ("srt"));
+  g_ptr_array_add (argv, g_strdup ("-c"));
+  g_ptr_array_add (argv, g_strdup ("subtitles"));
+/*
+  g_ptr_array_add (argv, g_strdup ("-l"));
+  g_ptr_array_add (argv, g_strdup ("fr"));
+*/
+  output = g_strconcat (input, ".ogg", NULL);
+  g_ptr_array_add (argv, g_strdup ("-o"));
+  g_ptr_array_add (argv, output);
+
+  g_ptr_array_add (argv, g_strdup (input));
 
   g_ptr_array_add (argv, NULL);
 
@@ -125,25 +166,85 @@ ogmrip_oggz_init (OGMRipOggz *oggz)
 {
 }
 
-static gint
-ogmrip_oggz_run (OGMJobSpawn *spawn)
+static void
+ogmrip_oggz_add_subp (OGMJobQueue *queue, const gchar *input)
 {
   OGMJobSpawn *child;
   gchar **argv;
+
+  argv = ogmrip_kate_command (input);
+  if (argv)
+  {
+    child = ogmjob_exec_newv (argv);
+    ogmjob_container_add (OGMJOB_CONTAINER (queue), child);
+    g_object_unref (child);
+  }
+}
+
+static void
+ogmrip_oggz_add_subp_stream (OGMRipContainer *oggz, OGMRipCodec *codec,
+    guint demuxer, gint language, OGMJobQueue *queue)
+{
+  ogmrip_oggz_add_subp (queue, ogmrip_codec_get_output (codec));
+}
+
+static void
+ogmrip_oggz_add_subp_file (OGMRipContainer *oggz, OGMRipFile *file, OGMJobQueue *queue)
+{
+  gchar *filename;
+
+  filename = ogmrip_file_get_filename (file);
+  if (filename && ogmrip_file_get_type (file) == OGMRIP_FILE_TYPE_SUBP)
+    ogmrip_oggz_add_subp (queue, filename);
+  g_free (filename);
+}
+
+static void
+ogmrip_oggz_remove_subp (OGMRipContainer *oggz, OGMRipCodec *codec,
+    guint demuxer, gint language, OGMJobQueue *queue)
+{
+  gchar *filename;
+
+  filename = g_strconcat (ogmrip_codec_get_output (codec), ".ogg", NULL);
+  g_unlink (filename);
+  g_free (filename);
+}
+
+static gint
+ogmrip_oggz_run (OGMJobSpawn *spawn)
+{
+  OGMJobSpawn *queue, *child;
+  gchar **argv;
   gint result;
 
+  queue = ogmjob_queue_new ();
+  ogmjob_container_add (OGMJOB_CONTAINER (spawn), queue);
+  g_object_unref (queue);
+
+  if (have_kate_enc)
+  {
+    ogmrip_container_foreach_subp (OGMRIP_CONTAINER (spawn),
+        (OGMRipContainerCodecFunc) ogmrip_oggz_add_subp_stream, queue);
+    ogmrip_container_foreach_file (OGMRIP_CONTAINER (spawn),
+        (OGMRipContainerFileFunc) ogmrip_oggz_add_subp_file, queue);
+  }
+
   argv = ogmrip_oggz_command (OGMRIP_CONTAINER (spawn));
   if (!argv)
     return OGMJOB_RESULT_ERROR;
 
   child = ogmjob_exec_newv (argv);
   ogmjob_exec_add_watch_full (OGMJOB_EXEC (child), (OGMJobWatch) NULL, spawn, TRUE, FALSE, FALSE);
-  ogmjob_container_add (OGMJOB_CONTAINER (spawn), child);
+  ogmjob_container_add (OGMJOB_CONTAINER (queue), child);
   g_object_unref (child);
 
   result = OGMJOB_SPAWN_CLASS (ogmrip_oggz_parent_class)->run (spawn);
 
-  ogmjob_container_remove (OGMJOB_CONTAINER (spawn), child);
+  if (have_kate_enc)
+    ogmrip_container_foreach_subp (OGMRIP_CONTAINER (spawn),
+        (OGMRipContainerCodecFunc) ogmrip_oggz_remove_subp, queue);
+
+  ogmjob_container_remove (OGMJOB_CONTAINER (spawn), queue);
 
   return result;
 }
@@ -165,6 +266,7 @@ static gint formats[] =
 {
   OGMRIP_FORMAT_VORBIS,
   OGMRIP_FORMAT_THEORA,
+  -1,
   -1
 };
 
@@ -195,6 +297,16 @@ ogmrip_init_plugin (GError **error)
 
   g_free (fullname);
 
+  fullname = g_find_program_in_path ("kateenc");
+  have_kate_enc = fullname != NULL;
+  g_free (fullname);
+
+  if (have_kate_enc)
+  {
+    formats[2] = OGMRIP_FORMAT_SRT;
+    oggz_plugin.max_subp = G_MAXINT;
+  }
+
   oggz_plugin.type = OGMRIP_TYPE_OGGZ;
   oggz_plugin.formats = formats;
 
